/** Teardrop - wxEngineTree.cpp

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "wxEngineTree.h"

//Icons :
#include "folder_html.xpm"
#include "xmag.xpm"

#define ICON_CAT 0
#define ICON_DEFAULT 1

BEGIN_EVENT_TABLE(wxEngineTree, wxTreeCtrl)
    EVT_RIGHT_DOWN(wxEngineTree::OnPopupMenu)
    EVT_MENU(wxID_NEW, wxEngineTree::OnNewEngine)
    EVT_MENU(wxID_DELETE, wxEngineTree::OnDelete)
    EVT_MENU(wxID_UP, wxEngineTree::OnMoveUp)
    EVT_MENU(wxID_DOWN, wxEngineTree::OnMoveDown)
    EVT_MENU(wxID_HOME, wxEngineTree::OnNewCategory) //Not well choosen id, but...
END_EVENT_TABLE()

wxEngineTree::wxEngineTree(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, wxFrame* frame, const wxValidator& validator, const wxString& name):
    wxTreeCtrl(parent,id,pos,size,style,validator,name) {
    fra = frame;
    Create(frame);
        
    m_ddMenu = new wxMenu();
    m_ddMenu->Append(wxID_HOME,_("New Category"));
    m_ddMenu->Append(wxID_NEW,_("New Engine"));
    m_ddMenu->Append(wxID_DELETE,_("Remove"));
    m_ddMenu->Append(wxID_UP,_("Move Up"));
    m_ddMenu->Append(wxID_DOWN,_("Move Down"));
}

wxEngineTree::~wxEngineTree() {
}

void wxEngineTree::Create(wxFrame* frame) {
        AddRoot(_("Categories"));
    
    //This wxImageList is not destructed as wxwidgets does it by itself.
    wxImageList *wi = new wxImageList(16,16);
    
    wi->Add(wxIcon(folder_html_xpm)); // Will be the number 0 ICON_CAT
    wi->Add(wxIcon(xmag_xpm));   // Will be the number 1 ICON_DEFAULT
    AssignImageList(wi);
    
    //xmlFile xf;
    string config_path;

    // In C++, if the first condition is false, the second isn't tested. So it's valid.
    if (selectFile::find("config",".xml",config_path) && xf.openFile(config_path + "config.xml")) {
        xmlNode *cat = xf.findChildByName(NULL,"category");
        while (cat) {
            wxTreeItemId top = AppendItem(GetRootItem(),toWXString(xf.getAttributeValueByName(cat,"name")),ICON_CAT);
            xmlNode *cur = xf.findChildByName(cat,"engine");
            while (cur) {
                string name = xf.getAttributeValueByName(cur,"name");
                string path;
                if (selectFile::find(name,".xml",path)) {
                    string file = path + name + ".xml";
                    string icon = path + name + ".png";
                    int iconNum = ICON_DEFAULT;
                    if (wxFile::Exists(toWXString(icon))) {
                        wxBitmap bt(wxImage(toWXString(icon)));
                        wxIcon ic;
                        ic.CopyFromBitmap(bt);
                        iconNum = wi->Add(ic);
                    }
                    xmlEngine xe;
                    if (xe.openEngine(name)) {
                        string realname = xe.getName();
                        myList[realname] = name;
                        AppendItem(top,toWXString(realname), iconNum);
                    } else if (frame) frame->SetStatusText(_("A file was not found ! Please verify your configuration !"));
                }
                cur = xf.findNodeByName(cur->next,"engine");
            }
            cat = xf.findNodeByName(cat->next,"category");
        }
    } else {
        newConfigXML();
        if (!selectFile::find("config",".xml",config_path) || !xf.openFile(config_path + "config.xml"))
            if (frame) frame->SetStatusText(_("Couldn't open configuration file !!"));
    }
    
    wxTreeItemIdValue  cookie;
    //SelectItem( GetFirstChild( GetRootItem() , cookie ) );
}

list<string> wxEngineTree::getSelectedName() {
    list<string> out;
    wxArrayTreeItemIds ids;
    GetSelections(ids);
    for(unsigned int i = 0; i < ids.GetCount(); ++i) {
        //A category is selected, so every children should be added
        if (myList[toSTLString(GetItemText(ids[i]))] == "") {
            wxTreeItemIdValue cook;
            wxTreeItemId id = GetFirstChild(ids[i],cook);
            while (id.IsOk()) {
                out.push_back(myList[toSTLString(GetItemText(id))]);
                id = GetNextChild(ids[i],cook);
            }
        } else if (!IsSelected(GetItemParent(ids[i])))
                out.push_back(myList[toSTLString(GetItemText(ids[i]))]);
    }
    return out;
}

void wxEngineTree::addEngine(tdParam & std, string url, string c) {
    getHttp gh;
    gh.setParam(std);
    string path = selectFile::getHomeDirectory();
    
    string::size_type pos = url.find_last_of("/");
    if (pos != string::npos) {
        string eng = url.substr(pos+1);
        if (eng.length() < 5) return;
        string fname = eng.substr(0,eng.length()-4);
        string file = path + eng;

        //If XML retrieval fails, we abort there.
        if (!getFile(gh,url,file)) return;
        
        file = file.substr(0,file.length()-4) + ".png";
        url = url.substr(0,url.length()-4) + ".png";
        getFile(gh,url,file);
        
        if (getEngineNode(c,fname) == NULL) {
            xmlNode *cat = getCategoryNode(c);
            if (cat) {
                xmlNode *nd = xmlNewChild(cat,NULL,(const xmlChar*) "engine",NULL);
                xmlNewProp(nd,(const xmlChar*) "name",(const xmlChar*) fname.c_str());
                saveConfigXML();
            }
        }
    }
}

void wxEngineTree::addCategory(string c) {
    xmlNode *nd = xmlNewChild(xf.root_element,NULL,(const xmlChar*) "category",NULL);
    xmlNewProp(nd,(const xmlChar*) "name",(const xmlChar*) c.c_str());
    saveConfigXML();
}

void wxEngineTree::removeEngine(string c, string name) {
    xmlNode *nd;
    do {
        nd = getEngineNode(c,name);
        if (nd) {
            xmlUnlinkNode(nd);
            xmlFreeNode(nd);
        }
    } while (nd);
    
    saveConfigXML();
}

void wxEngineTree::moveEngineUp(string c, string name) {
    xmlNode *nd = getEngineNode(c,name);
    if (nd) {
        xmlNode *del = xf.findPreviousNodeByName(nd->prev,"engine");
        if (del) {
            xmlAddNextSibling(nd,del);
            saveConfigXML();
        }
    }
}

void wxEngineTree::moveEngineDown(string c, string name) {
    xmlNode *nd = getEngineNode(c,name);
    if (nd) {
        xmlNode *del = xf.findNodeByName(nd->next,"engine");
        if (del) {
            xmlAddPrevSibling(nd,del);
            saveConfigXML();
        }
    }
}

void wxEngineTree::moveCategoryUp(string c) {
    xmlNode *nd = getCategoryNode(c);
    if (nd) {
        xmlNode *del = xf.findPreviousNodeByName(nd->prev,"category");
        if (del) {
            xmlAddNextSibling(nd,del);
            saveConfigXML();
        }
    }
}

void wxEngineTree::moveCategoryDown(string c) {
    xmlNode *nd = getCategoryNode(c);
    if (nd) {
        xmlNode *del = xf.findNodeByName(nd->next,"category");
        if (del) {
            xmlAddPrevSibling(nd,del);
            saveConfigXML();
        }
    }
}

xmlNode *wxEngineTree::getEngineNode(string & c, string & name) {
    xmlNode *cat = xf.findChildByName(NULL,"category");
    while (cat) {
        if (xf.getAttributeValueByName(cat,"name") == c) {
            xmlNode *nd = xf.findChildByName(cat,"engine");
            while(nd) {
                if (xf.getAttributeValueByName(nd,"name") == name) {
                    return nd;
                } else nd = xf.findNodeByName(nd->next,"engine");
            }
        }
        cat = xf.findNodeByName(cat->next,"category");
    }
    return NULL;
}

xmlNode *wxEngineTree::getCategoryNode(string & c) {
    xmlNode *cat = xf.findChildByName(NULL,"category");
    while (cat) {
        if (xf.getAttributeValueByName(cat,"name") == c) return cat;
        cat = xf.findNodeByName(cat->next,"category");
    }
    return NULL;
}

void wxEngineTree::removeCategory(string name) {
    xmlNode *cat = getCategoryNode(name);
    if (cat) {
        xmlUnlinkNode(cat);
        xmlFreeNode(cat);
        saveConfigXML();
    }
}

void wxEngineTree::OnPopupMenu(wxMouseEvent& event) {
    m_mSelect = event.GetPosition();
    int flags = 0;
    wxTreeItemId it = HitTest(m_mSelect, flags);
    UnselectAll();
    SelectItem(it);
    PopupMenu(m_ddMenu);
}

void wxEngineTree::OnNewEngine(wxCommandEvent& event) {
    int flags = 0;
    wxTreeItemId it = HitTest(m_mSelect, flags);
    if (myList[toSTLString(GetItemText(it))] != "") it = GetItemParent(it);
    string cat = toSTLString(GetItemText(it));
    wxTextEntryDialog txt(this, _("Where is the engine's definition located ?\n(See http://olivier.coupelon.free.fr/teardrop/engines_tou.php)"),_("Add a new engine"),_T(""),wxOK|wxCANCEL);
    if (txt.ShowModal() == wxID_OK) {
        if (!txt.GetValue().IsEmpty()) {
            addEngine(GetTdParam(),toSTLString(txt.GetValue()),cat);
            Reload();
        }
    }
}

void wxEngineTree::OnNewCategory(wxCommandEvent& event) {
    wxTextEntryDialog txt(this, _("Enter the name of the new category"),_("Add a new category"),_("New Category"),wxOK|wxCANCEL);
    if (txt.ShowModal() == wxID_OK) {
        string val = toSTLString(txt.GetValue());
        if (val != "" && getCategoryNode(val) == NULL)
            addCategory(val);
            Reload();
    }
}

void wxEngineTree::OnDelete(wxCommandEvent& event) {
    int flags = 0;
    wxTreeItemId it = HitTest(m_mSelect, flags);
    string name = toSTLString(GetItemText(it));
    if (myList[name] != "") {
        string cat = toSTLString(GetItemText(GetItemParent(it)));
        removeEngine(cat,myList[name]);
        Reload();
    } else if (GetChildrenCount(it) == 0) {
        removeCategory(name);
        Reload();
    }
}

void wxEngineTree::OnMoveUp(wxCommandEvent& event) {
    wxArrayTreeItemIds ids;
    size_t s;
    if ((s = GetSelections(ids)) != 0) {
        for (int i = 0; i < s; ++i) {
            string name = toSTLString(GetItemText(ids[i]));
            if (myList[name] != "") {
                //Moving up an engine
                string cat = toSTLString(GetItemText(GetItemParent(ids[i])));
                moveEngineUp(cat,myList[name]);
            } else {
                moveCategoryUp(name);
            }
        }
        Reload();
    }
}

void wxEngineTree::OnMoveDown(wxCommandEvent& event) {
    wxArrayTreeItemIds ids;
    size_t s;
    if ((s = GetSelections(ids)) != 0) {
        for (int i = s-1; i >= 0; --i) {
            string name = toSTLString(GetItemText(ids[i]));
            if (myList[name] != "") {
                //Moving down an engine
                string cat = toSTLString(GetItemText(GetItemParent(ids[i])));
                moveEngineDown(cat,myList[name]);
            } else {
                moveCategoryDown(name);
            }
        }
        Reload();
    }
}

void wxEngineTree::SetTdParam(tdParam & t) {
    tdp = t;
}

tdParam & wxEngineTree::GetTdParam() {
    return tdp;
}

void wxEngineTree::Reload() {
    // Saves the categories state :
    map<wxString,bool> tog;
    map<wxString,bool> sel;
    wxTreeItemIdValue cook;
    wxTreeItemId p = GetFirstChild(GetRootItem(),cook);
    while (p.IsOk()) {
        wxTreeItemId c = GetFirstChild(p,cook);
        while(c.IsOk()) {
            sel[GetItemText(c)] = IsSelected(c);
            c = GetNextSibling(c);
        }
        tog[GetItemText(p)] = IsExpanded(p);
        sel[GetItemText(p)] = IsSelected(p);
        p = GetNextSibling(p);
    }
    
    DeleteAllItems();
    Create(fra);
    
    // Restores the state of each known categories :
    p = GetFirstChild(GetRootItem(),cook);
    while (p.IsOk()) {
        wxTreeItemId c = GetFirstChild(p,cook);
        while(c.IsOk()) {
            if (sel.find(GetItemText(c)) != sel.end()) {
                if (sel[GetItemText(c)]) SelectItem(c);
            }
            c = GetNextSibling(c);
        }
        if (tog.find(GetItemText(p)) != tog.end()) {
            if (sel[GetItemText(p)]) SelectItem(p);
            if (tog[GetItemText(p)]) Expand(p);
        }
        p = GetNextSibling(p);
    }
}

void wxEngineTree::saveConfigXML() {
    string out = selectFile::getHomeDirectory() + "config.xml";
    xf.saveDocTo(out);
}

void wxEngineTree::newConfigXML() {
    string out = selectFile::getHomeDirectory() + "config.xml";
    ofstream outfile(out.c_str(),std::ios_base::binary);
    if (outfile.good()) {
        outfile << newconfig_file;
        outfile.close();
    }
}

bool wxEngineTree::getFile(getHttp & gh, string input, string output) {
     address a;
     a.url = input;
     if (gh.getFile(a,output)) return true;
     //Win32 CURL seems not to handle file:// protocol
     if (input.find("file://",0) == (string::size_type) 0)
        input = input.substr(0,7);
        
     ifstream infile(input.c_str(), std::ios_base::binary);
     if (infile.good()) {
         ofstream outfile(output.c_str(), std::ios_base::binary);
         if (outfile.good()) {
             outfile << infile.rdbuf();
             outfile.close();
             return true;
         }
         infile.close();
     }
     return false;
}

void wxEngineTree::UpdateEngines() {
    xmlNode *cat = xf.findChildByName(NULL,"category");
    threadPool<enginesUpdater> tp(false,1);
    int max = 0;
    while (cat) {
        string category = xf.getAttributeValueByName(cat,"name");
        xmlNode *cur = xf.findChildByName(cat,"engine");
        while (cur) {
            string name = xf.getAttributeValueByName(cur,"name");
            xmlEngine xe;
            if (xe.openEngine(name)) {
                if (xe.shouldUpdate()) {
                    string url = xe.getUpdateUrl();
                    if (url == "") url = TEARDROP_ENGINES_URL + name + ".xml";
                    max++;
                    tp.createThread(new enginesUpdater(url,category,GetTdParam(),this));
                }
            }
            cur = xf.findNodeByName(cur->next,"engine");
        }
        cat = xf.findNodeByName(cat->next,"category");
    }

    wxProgressDialog dialog(_("Updating engines definitions"), _("Please wait while engines are being updated..."), max+1,this, wxPD_AUTO_HIDE | wxPD_APP_MODAL);
    dialog.Update(0);
    while (tp.isAlive()) {
        wxMilliSleep(500);
        dialog.Update(tp.howManyFinished(),_T(""));
    }
    
    tp.joinAll();
    
    wxString msg;
    msg.Printf(_("%d engines have been updated !"), max);
    fra->SetStatusText(msg);
    Reload();
    
    dialog.Update(max+1);
}
