/** Teardrop - wxEngineTree

@brief This class represents and generates the engines tree.

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 19/08/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef ENGINELIST_H
#define ENGINELIST_H

#include <map>
#include <vector>
#include <list>
#include <fstream>

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlIO.h>

#include <wx/wx.h>
#include <wx/treectrl.h>
#include <wx/icon.h>
#include <wx/image.h>
#include <wx/bitmap.h>
#include <wx/imaglist.h>
#include <wx/file.h>
#include <wx/progdlg.h>

#include "xmlFile.h"
#include "xmlEngine.h"
#include "stringsConversions.h"
#include "selectFile.h"
#include "newconfig_file.h"

#define TEARDROP_ENGINES_URL "http://olivier.coupelon.free.fr/teardrop/download/engines/"

class wxEngineTree : public wxTreeCtrl {
    private:
    map<string,string> myList;
    wxFrame *fra;
    wxMenu *m_ddMenu;
    wxPoint m_mSelect;
    tdParam tdp;
    xmlFile xf;
    
    public:
    /**
    The constructor of this engine list.
    @param the wxTreeCtrl which should be filled by the engine list
    */
    wxEngineTree(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS, wxFrame* frame = NULL, const wxValidator& validator = wxDefaultValidator, const wxString& name = _T("listCtrl"));
    ~wxEngineTree();
    void Create(wxFrame*);
    /**
    Returns the name of the selected engines. If a category is selected,
    every engines belonging to it are added.
    */
    list<string> getSelectedName();
    void addEngine(tdParam &, string, string);
    void addCategory(string);
    xmlNode *getEngineNode(string &, string &);
    xmlNode *getCategoryNode(string &);
    void OnPopupMenu(wxMouseEvent& event);
    void OnNewEngine(wxCommandEvent& event);
    void OnNewCategory(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnMoveUp(wxCommandEvent& event);
    void OnMoveDown(wxCommandEvent& event);
    void SetTdParam(tdParam &);
    tdParam & GetTdParam();
    void removeCategory(string name);
    void removeEngine(string c, string name);
    void Reload();
    void moveEngineUp(string c, string name);
    void moveEngineDown(string c, string name);
    void moveCategoryUp(string c);
    void moveCategoryDown(string c);
    void saveConfigXML();
    void newConfigXML();
    bool getFile(getHttp &, string, string);
    void UpdateEngines();

    DECLARE_EVENT_TABLE()
};

class enginesUpdater {
private:
    string url;
    string category;
    tdParam tdp;
    wxEngineTree *parent;
public:
    enginesUpdater(string u,string c, const tdParam & t,wxEngineTree *p) {
        url = u;
        category = c;
        tdp = t;
        parent = p;
    }
	void operator()() { parent->addEngine(tdp,url,category); }
};

#endif
