/** Teardrop - wxPrefDialog.cpp

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "wxPrefDialog.h"

IMPLEMENT_CLASS( wxPrefDialog, wxDialog )

BEGIN_EVENT_TABLE( wxPrefDialog, wxDialog )
    EVT_BUTTON( ID_RESET, wxPrefDialog::OnResetClick )
END_EVENT_TABLE()

wxPrefDialog::wxPrefDialog():wxDialog() {
}

wxPrefDialog::wxPrefDialog(wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style):
wxDialog(parent, id, caption, pos, size, style) {
    Create(parent, id, caption, pos, size, style);
}

bool wxPrefDialog::Create(wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style) {
    CreateControls();
    return true;
}

void wxPrefDialog::CreateControls() {
    wxBoxSizer* topSizer = new wxBoxSizer(wxVERTICAL);
    this->SetSizer(topSizer);
    // A second box sizer to give more space around the controls
    wxBoxSizer* boxSizer = new wxBoxSizer(wxVERTICAL);
    topSizer->Add(boxSizer, 0, wxEXPAND, 5);

    //boxSizer->Add(5, 5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticText* browserLabel = new wxStaticText ( this, -1, _("Browser (Use \"%b\" in place of the URL) :"));
    boxSizer->Add(browserLabel, 0, wxALIGN_LEFT|wxALL, 5);

    wxTextCtrl* browserCtrl = new wxTextCtrl ( this, ID_BROWSER, m_browser);
    boxSizer->Add(browserCtrl, 0, wxGROW|wxALL, 5);
    
    wxStaticLine* line1 = new wxStaticLine ( this, -1,
        wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    boxSizer->Add(line1, 0, wxGROW|wxALL, 5);
    
    wxBoxSizer* proxySizer = new wxBoxSizer(wxHORIZONTAL);
    boxSizer->Add(proxySizer, 0, wxEXPAND);
    
    wxStaticText* proxyAddressLabel = new wxStaticText ( this, -1, _("Proxy :"), wxDefaultPosition, wxDefaultSize, 0 );
    wxTextCtrl* proxyAddressCtrl = new wxTextCtrl ( this, ID_PROXY_ADDRESS, m_proxyAddress, wxDefaultPosition, wxSize(300,-1));
    wxStaticText* proxyPortLabel = new wxStaticText ( this, -1, _("Port :"), wxDefaultPosition, wxDefaultSize, 0 );
    wxTextCtrl* proxyPortCtrl = new wxTextCtrl ( this, ID_PROXY_PORT, m_proxyPort);
    proxySizer->Add(proxyAddressLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    proxySizer->Add(proxyAddressCtrl, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    proxySizer->Add(proxyPortLabel, 0, wxGROW|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    proxySizer->Add(proxyPortCtrl, 0, wxGROW|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    wxBoxSizer* proxy2Sizer = new wxBoxSizer(wxHORIZONTAL);
    boxSizer->Add(proxy2Sizer);
    
    wxRadioButton* httpRad = new wxRadioButton(this, ID_PROXY_HTTP, _("HTTP"),wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
    wxRadioButton* Socks5Rad = new wxRadioButton(this, ID_PROXY_SOCKS5, _("SOCKS v5"));
    proxy2Sizer->Add(httpRad, 0, wxGROW|wxALL, 5);
    proxy2Sizer->Add(Socks5Rad, 0, wxGROW|wxALL, 5);
    
    wxStaticLine* line2 = new wxStaticLine ( this, -1,
        wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );    
    boxSizer->Add(line2, 0, wxGROW|wxALL, 5);
    
    wxBoxSizer* threadSizer = new wxBoxSizer(wxHORIZONTAL);
    boxSizer->Add(threadSizer);
    wxStaticText* threadLabel = new wxStaticText ( this, -1, _("Maximum number of concurrent threads :"), wxDefaultPosition, wxDefaultSize, 0 );
    wxTextCtrl* threadCtrl = new wxTextCtrl ( this, ID_THREADNUM, m_threadMax);
    threadSizer->Add(threadLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    threadSizer->Add(threadCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    wxBoxSizer* timeoutSizer = new wxBoxSizer(wxHORIZONTAL);
    boxSizer->Add(timeoutSizer);
    wxStaticText* timeoutLabel = new wxStaticText ( this, -1, _("Network connections timeout (seconds) :"), wxDefaultPosition, wxDefaultSize, 0 );
    wxTextCtrl* timeoutCtrl = new wxTextCtrl ( this, ID_TIMEOUT, m_timeout);
    timeoutSizer->Add(timeoutLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    timeoutSizer->Add(timeoutCtrl, 0, wxALIGN_LEFT|wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    wxBoxSizer* okCancelBox = new wxBoxSizer(wxHORIZONTAL);
    boxSizer->Add(okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    // The Reset button
    wxButton* reset = new wxButton( this, ID_RESET, wxT("&Reset"), wxDefaultPosition, wxDefaultSize, 0 );
    okCancelBox->Add(reset, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    
    // The OK button
    wxButton* ok = new wxButton ( this, wxID_OK, wxT("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    okCancelBox->Add(ok, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    
    // The Cancel button
    wxButton* cancel = new wxButton ( this, wxID_CANCEL, wxT("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    okCancelBox->Add(cancel, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    
    FindWindow(ID_BROWSER)->SetValidator(wxGenericValidator(& m_browser));
    FindWindow(ID_PROXY_ADDRESS)->SetValidator(wxGenericValidator(& m_proxyAddress));
    FindWindow(ID_PROXY_PORT)->SetValidator(wxTextValidator(wxFILTER_NUMERIC, & m_proxyPort));
    FindWindow(ID_PROXY_HTTP)->SetValidator(wxGenericValidator(& m_proxyHttp));
    FindWindow(ID_PROXY_SOCKS5)->SetValidator(wxGenericValidator(& m_proxySocks5));
    FindWindow(ID_THREADNUM)->SetValidator(wxTextValidator(wxFILTER_NUMERIC, & m_threadMax));
    FindWindow(ID_TIMEOUT)->SetValidator(wxTextValidator(wxFILTER_NUMERIC, & m_timeout));
    
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();
    
}

void wxPrefDialog::OnResetClick(wxCommandEvent &event) {
    TransferDataToWindow();
}
