/** Teardrop - row.h

@brief This class is used to represent a row (tuple) extracted by a query.

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 04/04/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef ROW_H
#define ROW_H

#include <iostream>
#include <map>
#include <list>

using namespace std;

struct compstr {
    bool operator()(const string s1, const string s2) const
    {
        return s1 < s2;
    }
};

class row {
public:
  row(int n = 0);
  row(int, string);
  int getNum() const;
  void setNum(int);
  list<string> getEngine() const;
  void addField(string, string);
  void addComparable(string);
  void addEngine(string);
  void addEngine(list<string>);
  map<string, string, compstr> & getFields();
  string getField(string s) const;
  void toString(bool n = true, string sep = " ");
  bool operator==(const row &) const;

private:
  int num;
  list<string> engine;
  list<string> comparable;
  map<string, string, compstr> fields;
};

#endif
