/** Teardrop - MyApp.cpp

# Copyright (C) 2006 Olivier COUPELON
@brief The main loop of the GUI

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 26/02/2006

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <wx/wx.h>
#include <wx/intl.h>
#include "MyFrame.h"

#define wxUSE_UNICODE 1

// Declare the application class
class MyApp : public wxApp {
public:
    // Called on application startup
    bool OnInit();
protected:
        wxLocale m_locale;
};

// Give wxWidgets the means to create a MyApp object
IMPLEMENT_APP(MyApp)

bool MyApp::OnInit() {
    /* Create the Teardrop personal directory if it doesn't exist */
    string path = selectFile::getHomeDirectory();
    umask(0);
    #ifdef __WXMSW__
    mkdir(path.c_str());
    #else
    mkdir(path.c_str(),0755);
    #endif 
     
    m_locale.Init(wxLANGUAGE_DEFAULT,wxLOCALE_LOAD_DEFAULT | wxLOCALE_CONV_ENCODING);
    #ifdef TEARDROP_DATADIR
        wxLocale::AddCatalogLookupPathPrefix(wxT(TEARDROP_DATADIR) wxT("/"));
    #endif
    m_locale.AddCatalog(wxT("teardrop"));
    
    //Initialise every image handlers
    wxInitAllImageHandlers();
    // Create the main application window
    MyFrame *frame = new MyFrame(NULL, wxID_ANY,wxT("Teardrop"),wxDefaultPosition,wxDefaultSize,0);
    // Show it
    frame->Show(true);
    // Start the event loop
    return true;
}
